// โหลดไฟล์ .env เพื่อดึงค่าคอนฟิกต่าง ๆ
require('dotenv').config();
const express = require('express');
const session = require('express-session');
const bodyParser = require('body-parser');
const bcrypt = require('bcryptjs');
const mysql = require('mysql2');

const app = express();
const port = process.env.PORT;

// ตั้งค่า view engine เป็น EJS
app.set('view engine', 'ejs');
app.use(bodyParser.urlencoded({ extended: false }));
app.use(express.static('public'));

// ตั้งค่าระบบ session
app.use(session({
  secret: process.env.SESSION_SECRET || 'default_secret_key',
  resave: false,
  saveUninitialized: false
}));

// เชื่อมต่อฐานข้อมูล MySQL
const connection = mysql.createConnection({
  host: process.env.DB_HOST || 'localhost',
  user: process.env.DB_USER || 'root',
  password: process.env.DB_PASS || '',
  database: process.env.DB_NAME || 'cloud_db_threads',
  port: process.env.DB_PORT || 3306
});

connection.connect(err => {
  if (err) {
    console.error('Error connecting to MySQL:', err);
    process.exit(1);
  }
  console.log('Connected to MySQL Successfully');
});

// Middleware ที่ใช้เช็คว่าผู้ใช้ล็อกอินหรือยัง
function isAuthenticated(req, res, next) {
  if (req.session.userId) return next();
  res.redirect('/login');
}

// หน้าแรก: พาไป dashboard ทันที ( User แบบ Guest สามารถเข้าได้)
app.get('/', (req, res) => {
  res.redirect('/dashboard');
});

// หน้า register
app.get('/register', (req, res) => {
  res.render('register', { error: null });
});

// ดำเนินการ register ผู้ใช้ใหม่
app.post('/register', async (req, res) => {
  const { username, email, password } = req.body;
  try {
    const hashedPassword = await bcrypt.hash(password, 10);
    connection.query(
      'INSERT INTO users (username, email, password) VALUES (?, ?, ?)',
      [username, email, hashedPassword],
      (err) => {
        if (err) {
          console.error(err);
          if (err.code === 'ER_DUP_ENTRY') {
            return res.render('register', { error: 'ชื่อผู้ใช้นี้ถูกใช้ไปแล้ว' });
          }
          return res.render('register', { error: 'เกิดข้อผิดพลาดในการลงทะเบียน' });
        }
        res.redirect('/login');
      }
    );
  } catch (error) {
    console.error(error);
    res.render('register', { error: 'เกิดข้อผิดพลาดระหว่างดำเนินการลงทะเบียน' });
  }
});

// หน้า login
app.get('/login', (req, res) => {
  res.render('login', { error: null });
});

// ดำเนินการ login
app.post('/login', (req, res) => {
  const { userInput, password } = req.body;
  connection.query(
    'SELECT * FROM users WHERE username = ? OR email = ?',
    [userInput, userInput],
    async (err, results) => {
      if (err) return res.render('login', { error: 'เกิดข้อผิดพลาด' });
      if (results.length === 0) return res.render('login', { error: 'ไม่พบผู้ใช้งาน' });
      const user = results[0];
      const match = await bcrypt.compare(password, user.password);
      if (match) {
        req.session.userId = user.id;
        req.session.username = user.username;
        res.redirect('/dashboard');
      } else {
        res.render('login', { error: 'รหัสผ่านไม่ถูกต้อง' });
      }
    }
  );
});

// หน้า dashboard แสดงรายการ threads ทั้งหมด
app.get('/dashboard', (req, res) => {
  const searchQuery = req.query.search;
  let sql = `
    SELECT threads.*, users.username AS ownerName
    FROM threads
    JOIN users ON threads.user_id = users.id
  `;
  const params = [];
  if (searchQuery) {
    sql += `WHERE threads.thread_name LIKE ? OR threads.description LIKE ?`;
    params.push(`%${searchQuery}%`, `%${searchQuery}%`);
  }
  sql += ' ORDER BY threads.id DESC';
  connection.query(sql, params, (err, threads) => {
    if (err) return res.send('เกิดข้อผิดพลาดในการดึงข้อมูล threads');
    res.render('dashboard', {
      username: req.session.username,
      userId: req.session.userId,
      threads,
      query: searchQuery || ''
    });
  });
});

// หน้าเพิ่ม thread ใหม่
app.get('/thread/create', isAuthenticated, (req, res) => {
  res.render('thread-create', { username: req.session.username });
});

// บันทึก thread ใหม่ลงฐานข้อมูล
app.post('/thread/create', isAuthenticated, (req, res) => {
  const { thread_name, description } = req.body;
  connection.query(
    'INSERT INTO threads (user_id, thread_name, description) VALUES (?, ?, ?)',
    [req.session.userId, thread_name, description],
    (err) => {
      if (err) return res.send('เกิดข้อผิดพลาดในการสร้าง thread');
      res.redirect('/dashboard');
    }
  );
});

// หน้าแก้ไข thread
app.get('/thread/edit/:id', isAuthenticated, (req, res) => {
  const threadId = req.params.id;
  connection.query(
    'SELECT * FROM threads WHERE id = ? AND user_id = ?',
    [threadId, req.session.userId],
    (err, results) => {
      if (err || results.length === 0) return res.send('ไม่พบ thread ที่ต้องการแก้ไข');
      res.render('thread-edit', {
        username: req.session.username,
        thread: results[0]
      });
    }
  );
});

// อัปเดตข้อมูล thread ที่แก้ไข
app.post('/thread/edit/:id', isAuthenticated, (req, res) => {
  const threadId = req.params.id;
  const { thread_name, description } = req.body;
  connection.query(
    'UPDATE threads SET thread_name = ?, description = ? WHERE id = ? AND user_id = ?',
    [thread_name, description, threadId, req.session.userId],
    (err) => {
      if (err) return res.send('เกิดข้อผิดพลาดในการอัปเดต thread');
      res.redirect('/dashboard');
    }
  );
});

// ลบ thread
app.get('/thread/delete/:id', isAuthenticated, (req, res) => {
  const threadId = req.params.id;
  connection.query(
    'DELETE FROM threads WHERE id = ? AND user_id = ?',
    [threadId, req.session.userId],
    (err) => {
      if (err) return res.send('เกิดข้อผิดพลาดในการลบ thread');
      res.redirect('/dashboard');
    }
  );
});

// หน้าแสดง thread พร้อมคอมเมนต์
app.get('/thread/:id', (req, res) => {
  const threadId = req.params.id;

  connection.query(
    `SELECT threads.*, users.username AS ownerName 
     FROM threads 
     JOIN users ON threads.user_id = users.id 
     WHERE threads.id = ?`,
    [threadId],
    (err, results) => {
      if (err || results.length === 0) return res.send('ไม่พบ thread ที่ต้องการ');

      const thread = results[0];

      connection.query(
        `SELECT comments.*, users.username AS commentUser, users.email 
         FROM comments 
         JOIN users ON comments.user_id = users.id 
         WHERE comments.thread_id = ? 
         ORDER BY comments.comment_id DESC`,  // ชื่อคอลัมน์ถูกต้อง
        [threadId],
        (err2, comments) => {
          if (err2) return res.send('เกิดข้อผิดพลาดในการดึงคอมเมนต์');

          res.render('thread-view', {
            username: req.session.username,
            userId: req.session.userId,
            thread,
            comments
          });
        }
      );
    }
  );
});

// เพิ่มคอมเมนต์ใหม่
app.post('/thread/:id/comment', isAuthenticated, (req, res) => {
  const threadId = req.params.id;
  const { comment } = req.body;
  connection.query(
    'INSERT INTO comments (thread_id, user_id, comment) VALUES (?, ?, ?)',
    [threadId, req.session.userId, comment],
    (err) => {
      if (err) return res.send('เกิดข้อผิดพลาดในการเพิ่มคอมเมนต์');
      res.redirect(`/thread/${threadId}`);
    }
  );
});

// ลบคอมเมนต์
app.get('/comment/delete/:id', isAuthenticated, (req, res) => {
  const commentId = req.params.id;

   connection.query(
    'SELECT * FROM comments WHERE comment_id = ? AND user_id = ?',
    [commentId, req.session.userId],
    (err, results) => {
      if (err || results.length === 0) return res.send('ไม่พบคอมเมนต์ที่ต้องการลบ');

      const comment = results[0];
      connection.query(
        'DELETE FROM comments WHERE comment_id = ?', 
        [commentId], 
        (err2) => {
          if (err2) return res.send('เกิดข้อผิดพลาดในการลบคอมเมนต์');
          res.redirect(`/thread/${comment.thread_id}`);
        }
      );
    }
  );
});


// ดึงคอมเมนต์ล่าสุดจากทุกกระทู้
app.get('/api/comments/latest',isAuthenticated, (req, res) => {
  const sql = `
    SELECT c.comment, u.id AS user_id, u.email, u.username, c.created_at
    FROM comments c
    JOIN users u ON c.user_id = u.id
    ORDER BY c.created_at DESC
    LIMIT 10
  `;
  connection.query(sql, (err, results) => {
    if (err) {
      console.error('ดึงคอมเมนต์ล่าสุดล้มเหลว:', err);
      return res.status(500).json({ error: 'ดึงคอมเมนต์ไม่สำเร็จ' });
    }
    res.json(results);
  });
});

// ออกจากระบบ
app.get('/logout', (req, res) => {
  req.session.destroy();
  res.redirect('/dashboard');
});

// เริ่มต้นเซิร์ฟเวอร์
app.listen(port, '0.0.0.0', () => {
  console.log(`Server เปิดอยู่ที่ http://localhost:${port}`);
});
